/***************************************************************************
                          DistributedGrid_3D.h  -  description
                             -------------------
    begin                : January 15, 2018
    copyright            : (C) 2018 by Tomas Oberhuber
    email                : tomas.oberhuber@fjfi.cvut.cz
 ***************************************************************************/

/* See Copyright Notice in tnl/Copyright */

#pragma once

#include <TNL/Config/ConfigDescription.h>
#include <TNL/Meshes/Grid.h>
#include <TNL/Meshes/DistributedMeshes/DistributedGrid_Base.h>

namespace TNL {
namespace Meshes { 
namespace DistributedMeshes {

enum Directions3D { West = 0 , East = 1 , North = 2, South=3, Top =4, Bottom=5, 
                  NorthWest=6, NorthEast=7, SouthWest=8, SouthEast=9,
                  BottomWest=10,BottomEast=11,BottomNorth=12,BottomSouth=13,
                  TopWest=14,TopEast=15,TopNorth=16,TopSouth=17,
                  BottomNorthWest=18,BottomNorthEast=19,BottomSouthWest=20,BottomSouthEast=21,
                  TopNorthWest=22,TopNorthEast=23,TopSouthWest=24,TopSouthEast=25
                  };


template< typename RealType, typename Device, typename Index >
class DistributedMesh<Grid< 3, RealType, Device, Index >> : public DistributedGrid_Base<3, RealType, Device, Index >
{

    public:

      //using typename DistributedGrid_Base<3, RealType, Device, Index >::IndexType;
      //using typename DistributedGrid_Base<3, RealType, Device, Index >::GridType;
      //using typename DistributedGrid_Base<3, RealType, Device, Index >::PointType;
      //using typename DistributedGrid_Base<3, RealType, Device, Index >::CoordinatesType;
      typedef typename DistributedGrid_Base<3, RealType, Device, Index >::CoordinatesType CoordinatesType;
      typedef typename DistributedGrid_Base<3, RealType, Device, Index >::IndexType IndexType;
      typedef typename DistributedGrid_Base<3, RealType, Device, Index >::GridType GridType;
      typedef typename DistributedGrid_Base<3, RealType, Device, Index >::PointType PointType;
    
      static void configSetup( Config::ConfigDescription& config );
      
      bool setup( const Config::ParameterContainer& parameters,
                  const String& prefix );

      template< typename CommunicatorType > 
      void setGlobalGrid( const GridType& globalGrid,
                          const CoordinatesType& overlap );
       
      void setupGrid( GridType& grid);
       
      String printProcessCoords() const;

      String printProcessDistr() const;

      const int* getNeighbors() const;
       
      void writeProlog( Logger& logger );

   private:

      int getRankOfProcCoord(int x, int y, int z) const;           
      int neighbors[26];

};

} // namespace DistributedMeshes
} // namespace Meshes
} // namespace TNL

#include <TNL/Meshes/DistributedMeshes/DistributedGrid_3D.hpp>
